import argparse
import os.path
import sys
from garmin_uploader.workflow import Workflow


def main():
    """
    CLI Entry point
    """
    base_dir = os.path.realpath(os.path.dirname(__file__))
    parser = argparse.ArgumentParser(
      formatter_class=argparse.RawDescriptionHelpFormatter,
      description='A script to upload .TCX, .GPX, and .FIT'
                  'files to the Garmin Connect web site.',
      epilog=open(os.path.join(base_dir, 'help.txt')).read(),
    )

    parser.add_argument(
        'paths',
        type=str,
        nargs='+',
        help='Path and name of file(s) to upload, list file name, or directory'
             'name containing fitness files.')
    parser.add_argument(
        '-a',
        '--name',
        dest='activity_name',
        type=str,
        help='Sets the activity name for the upload file. This option is'
             'ignored if multiple upload files are given.')
    parser.add_argument(
        '-t',
        '--type',
        dest='activity_type',
        type=str,
        help='Sets activity type for ALL files in filename list, except files'
             'described inside a csv list file.')
    parser.add_argument(
        '-u',
        '--username',
        dest='username',
        type=str,
        help='Garmin Connect user login')
    parser.add_argument(
        '-p',
        '--password',
        dest='password',
        type=str,
        help='Garmin Connect user password')
    parser.add_argument(
        '-v',
        '--verbose',
        dest='verbose',
        type=int,
        default=2,
        choices=[1, 2, 3, 4, 5],
        help='Verbose - select level of verbosity. 1=DEBUG(most verbose),'
             ' 2=INFO, 3=WARNING, 4=ERROR, 5= CRITICAL(least verbose).'
             ' [default=2]')

    # Run workflow with these options
    options = parser.parse_args()
    try:
        workflow = Workflow(**vars(options))
        workflow.run()
    except Exception as e:
        print('Error: {}'.format(e))
        return 1  # erroneous exit code

    return 0


if __name__ == '__main__':
    exit = main()
    sys.exit(exit)



import os.path
import glob
import csv
import time
import six
from garmin_uploader import (
    logger, VALID_GARMIN_FILE_EXTENSIONS, BINARY_FILE_FORMATS
)
from garmin_uploader.user import User
from garmin_uploader.api import GarminAPI, GarminAPIException


class Activity(object):
    """
    Garmin Connect Activity model
    """
    def __init__(self, path, name=None, type=None):
        self.id = None  # provided on upload
        self.path = path
        self.name = name
        self.type = type

    def __repr__(self):
        if self.id is None:
            out = self.name or self.filename
        else:
            out = '{} : {}'.format(self.id, self.name or self.filename)
        if six.PY3 and isinstance(out, bytes):
            return out.decode('utf8')
        else:
            return out

    @property
    def extension(self):
        extension = os.path.splitext(self.path)[1].lower()

        # Valid File extensions are .tcx, .fit, and .gpx
        if extension not in VALID_GARMIN_FILE_EXTENSIONS:
            raise Exception("Invalid File Extension")

        return extension

    @property
    def filename(self):
        """
        Garmin Connect web site does not comply with RFC 2231.
        urllib3 (used by the requests module) automatically detects non-ascii
        characters in filenames and generates the filename* header parameter
        (with asterisk - signifying that the filename has non-ascii characters)
        instead of the filename (without asterisk) header parameter.  Garmin
        Connect does not accept the asterisked version of filename and there
        is no way to tell urllib3 to not generate it.  The work-around for
        Garmin's noncompliant behavior (sending non-ascii characters with the
        non-asterisked filename parameter) is to always send an ascii encodable
        filename.  This is achieved by parsing out the non-ascii characters.
        """
        filename = os.path.basename(self.path)
        if six.PY3:
            return filename
        try:
            return filename.encode('ascii')
        except UnicodeEncodeError:
            return filename.decode('ascii', 'ignore')

    def open(self):
        """
        Open local activity file as a file descriptor
        """
        mode = self.extension in BINARY_FILE_FORMATS and 'rb' or 'r'
        return open(self.path, mode)

    def upload(self, user):
        """
        Upload an activity once authenticated
        """
        assert isinstance(user, User)
        assert user.session is not None

        api = GarminAPI()
        try:
            self.id, uploaded = api.upload_activity(user.session, self)
        except GarminAPIException as e:
            logger.warning('Upload failure: {}'.format(e))
            return False

        if uploaded:
            logger.info('Uploaded activity {}'.format(self))

            # Set activity name if specified
            if self.name:
                try:
                    api.set_activity_name(user.session, self)
                except GarminAPIException as e:
                    logger.warning('Activity name update failed: {}'.format(e))

            # Set activity type if specified
            if self.type:
                try:
                    api.set_activity_type(user.session, self)
                except GarminAPIException as e:
                    logger.warning('Activity type update failed: {}'.format(e))

        else:
            logger.info('Activity already uploaded {}'.format(self))

        return True


class Workflow():
    """
    Upload workflow:
     * List activities according to CLI args
     * Load user credentials
     * Authenticate user
     * Upload activities
    """

    def __init__(self, paths, username=None, password=None,
                 activity_type=None, activity_name=None, verbose=3):
        self.last_request = None
        logger.setLevel(level=verbose * 10)

        self.activity_type = activity_type
        self.activity_name = activity_name

        # Load activities
        self.activities = self.load_activities(paths)

        # Load user
        self.user = User(username, password)

    def load_activities(self, paths):
        """
        Load all activities files:
        Sort out file name args given on command line.  Figure out if they are
        fitness file names, directory names containing fitness files, or names
        of csv file lists.
        Also, expand file name wildcards, if necessary.  Check to see if files
        exist and if the file extension is valid.  Build lists of fitnes
        filenames, directories # which will be further searched for files, and
        list files.
        """

        def is_csv(filename):
            '''
            check to see if file exists and that the file
            extension is .csv
            '''
            extension = os.path.splitext(filename)[1].lower()
            return extension == '.csv' and os.path.isfile(filename)

        def is_activity(filename):
            '''
            check to see if file exists and that the extension is a
            valid activity file accepted by GC.
            '''
            if not os.path.isfile(filename):
                logger.warning("File '{}' does not exist. Skipping...".format(filename))  # noqa
                return False

            # Get file extension from name
            extension = os.path.splitext(filename)[1].lower()
            logger.debug("File '{}' has extension '{}'".format(filename, extension))  # noqa

            # Valid file extensions are .tcx, .fit, and .gpx
            if extension in VALID_GARMIN_FILE_EXTENSIONS:
                logger.debug("File '{}' extension '{}' is valid.".format(filename, extension))  # noqa
                return True
            else:
                logger.warning("File '{}' extension '{}' is not valid. Skipping file...".format(filename, extension))  # noqa
                return False

        valid_paths, csv_files = [], []
        for path in paths:
            path = os.path.realpath(path)
            if is_activity(path):
                # Use file directly
                valid_paths.append(path)

            elif is_csv(path):
                # Use file directly
                logger.info("List file '{}' will be processed...".format(path))
                csv_files.append(path)

            elif os.path.isdir(path):
                # Use files in directory
                # - Does not recursively drill into directories.
                # - Does not search for csv files in directories.
                valid_paths += [
                    f for f in glob.glob(os.path.join(path, '*'))
                    if is_activity(f)
                ]

        # Activity name given on command line only applies if a single filename
        # is given.  Otherwise, ignore.
        if len(valid_paths) != 1 and self.activity_name:
            logger.warning('-a option valid only when one fitness file given. Ignoring -a option.')  # noqa
            self.activity_name = None

        # Build activities from valid paths
        activities = [
           Activity(p, self.activity_name, self.activity_type)
           for p in valid_paths
        ]

        # Pull in file info from csv files and apppend activities
        for csv_file in csv_files:
            with open(csv_file, 'r') as csvfile:
                reader = csv.DictReader(csvfile)
                activities += [
                    Activity(row['filename'], row['name'], row['type'])
                    for row in reader
                    if is_activity(row['filename'])
                ]

        if len(activities) == 0:
            raise Exception('No valid files.')

        return activities

    def run(self):
        """
        Authenticated part of the workflow
        Simply login & upload every activity
        """
        if not self.user.authenticate():
            raise Exception('Invalid credentials')

        for activity in self.activities:
            self.rate_limit()
            activity.upload(self.user)

        logger.info('All done.')

    def rate_limit(self):
        min_period = 1
        if not self.last_request:
            self.last_request = 0.0

        wait_time = max(0, min_period - (time.time() - self.last_request))
        if wait_time <= 0:
            return
        time.sleep(wait_time)

        self.last_request = time.time()
        logger.info("Rate limited for %f" % wait_time)



import requests

import re
from garmin_uploader import logger

URL_HOSTNAME = 'https://connect.garmin.com/modern/auth/hostname'
URL_LOGIN = 'https://sso.garmin.com/sso/login'
URL_POST_LOGIN = 'https://connect.garmin.com/modern/'
URL_PROFILE = 'https://connect.garmin.com/modern/proxy/userprofile-service/socialProfile/'  # noqa
URL_HOST_SSO = 'sso.garmin.com'
URL_HOST_CONNECT = 'connect.garmin.com'
URL_SSO_SIGNIN = 'https://sso.garmin.com/sso/signin'
URL_UPLOAD = 'https://connect.garmin.com/modern/proxy/upload-service/upload'
URL_ACTIVITY_BASE = 'https://connect.garmin.com/modern/proxy/activity-service/activity'  # noqa
URL_ACTIVITY_TYPES = 'https://connect.garmin.com/modern/proxy/activity-service/activity/activityTypes' # noqa


class GarminAPIException(Exception):
    """
    An Exception occured in Garmin API
    """


class GarminAPI:
    """
    Low level Garmin Connect api connector
    """
    activity_types = None

    # This strange header is needed to get auth working
    common_headers = {
        'NK': 'NT',
    }

    def authenticate(self, username, password):
        """
        That's where the magic happens !
        Try to mimick a browser behavior trying to login
        on Garmin Connect as closely as possible
        Outputs a Requests session, loaded with precious cookies
        """
        # Use a valid Browser user agent
        # TODO: use several UA picked randomly
        session = requests.Session()
        session.headers.update({
            'User-Agent': 'Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:48.0) Gecko/20100101 Firefox/50.0',  # noqa
        })

        # Request sso hostname
        sso_hostname = None
        resp = session.get(URL_HOSTNAME)
        if not resp.ok:
            raise Exception('Invalid SSO first request status code {}'.format(resp.status_code))  # noqa
        sso_hostname = resp.json().get('host')

        # Load login page to get login ticket
        # Full parameters from Firebug, we have to maintain
        # Fuck this shit.
        # Who needs mandatory urls in a request parameters !
        params = {
            'clientId': 'GarminConnect',
            'connectLegalTerms': 'true',
            'consumeServiceTicket': 'false',
            'createAccountShown': 'true',
            'cssUrl': 'https://connect.garmin.com/gauth-custom-v1.2-min.css',
            'displayNameShown': 'false',
            'embedWidget': 'false',
            'gauthHost': 'https://sso.garmin.com/sso',
            'generateExtraServiceTicket': 'true',
            'generateNoServiceTicket': 'false',
            'generateTwoExtraServiceTickets': 'true',
            'globalOptInChecked': 'false',
            'globalOptInShown': 'true',
            'id': 'gauth-widget',
            'initialFocus': 'true',
            'locale': 'fr_FR',
            'locationPromptShown': 'true',
            'mfaRequired': 'false',
            'mobile': 'false',
            'openCreateAccount': 'false',
            'privacyStatementUrl': 'https://www.garmin.com/fr-FR/privacy/connect/',
            'redirectAfterAccountCreationUrl': 'https://connect.garmin.com/modern/',
            'redirectAfterAccountLoginUrl': 'https://connect.garmin.com/modern/',
            'rememberMeChecked': 'false',
            'rememberMeShown': 'true',
            'rememberMyBrowserChecked': 'false',
            'rememberMyBrowserShown': 'false',
            'service': 'https://connect.garmin.com/modern/',
            'showConnectLegalAge': 'false',
            'showPassword': 'true',
            'showPrivacyPolicy': 'false',
            'showTermsOfUse': 'false',
            'source': 'https://connect.garmin.com/signin/',
            'useCustomHeader': 'false',
            'webhost': sso_hostname,
        }
        res = session.get(URL_LOGIN, params=params)
        if res.status_code != 200:
            raise Exception('No login form')

        # Lookup for CSRF token
        csrf = re.search(r'<input type="hidden" name="_csrf" value="(\w+)" />', res.content.decode('utf-8'))  # noqa
        if csrf is None:
            raise Exception('No CSRF token')
        csrf_token = csrf.group(1)
        logger.debug('Found CSRF token {}'.format(csrf_token))

        # Login/Password with login ticket
        data = {
          'embed': 'false',
          'username': username,
          'password': password,
          '_csrf': csrf_token,
        }
        headers = {
          'Host': URL_HOST_SSO,
          'Referer': URL_SSO_SIGNIN,
        }
        res = session.post(URL_LOGIN, params=params, data=data,
                           headers=headers)
        if not res.ok:
            raise Exception('Authentification failed.')

        # Check we have sso guid in cookies
        if 'GARMIN-SSO-GUID' not in session.cookies:
            raise Exception('Missing Garmin auth cookie')

        # Try to find the full post login url in response
        regex = 'var response_url(\s+)= (\"|\').*?ticket=(?P<ticket>[\w\-]+)(\"|\')'  # noqa
        params = {}
        matches = re.search(regex, res.text)
        if not matches:
            raise Exception('Missing service ticket')
        params['ticket'] = matches.group('ticket')
        logger.debug('Found service ticket {}'.format(params['ticket']))

        # Second auth step
        # Needs a service ticket from previous response
        headers = {
            'Host': URL_HOST_CONNECT,
        }
        res = session.get(URL_POST_LOGIN, params=params, headers=headers)
        if res.status_code != 200 and not res.history:
            raise Exception('Second auth step failed.')

        # Check login
        res = session.get(URL_PROFILE)
        if not res.ok:
            raise Exception("Login check failed.")
        garmin_user = res.json()
        logger.info('Logged in as {}'.format(garmin_user['fullName']))

        return session

    def upload_activity(self, session, activity):
        """
        Upload an activity on Garmin
        Support multiple formats
        """
        assert activity.id is None

        # Upload file as multipart form
        files = {
            "file": (activity.filename, activity.open()),
        }
        url = '{}/{}'.format(URL_UPLOAD, activity.extension)
        res = session.post(url, files=files, headers=self.common_headers)

        # HTTP Status can either be OK or Conflict
        if res.status_code not in (200, 201, 409):
            if res.status_code == 412:
                logger.error('You may have to give explicit consent for uploading files to Garmin')  # noqa
            raise GarminAPIException('Failed to upload {}'.format(activity))

        response = res.json()['detailedImportResult']
        if len(response["successes"]) == 0:
            if len(response["failures"]) > 0:
                if response["failures"][0]["messages"][0]['code'] == 202:
                    # Activity already exists
                    return response["failures"][0]["internalId"], False
                else:
                    raise GarminAPIException(response["failures"][0]["messages"])  # noqa
            else:
                raise GarminAPIException('Unknown error: {}'.format(response))
        else:
            # Upload was successsful
            return response["successes"][0]["internalId"], True

    def set_activity_name(self, session, activity):
        """
        Update the activity name
        """
        assert activity.id is not None
        assert activity.name is not None

        url = '{}/{}'.format(URL_ACTIVITY_BASE, activity.id)
        data = {
            'activityId': activity.id,
            'activityName': activity.name,
        }
        headers = dict(self.common_headers)  # clone
        headers['X-HTTP-Method-Override'] = 'PUT'  # weird. again.
        res = session.post(url, json=data, headers=headers)
        if not res.ok:
            raise GarminAPIException('Activity name not set: {}'.format(res.content))  # noqa

    def load_activity_types(self):
        """
        Fetch valid activity types from Garmin Connect
        """
        # Only fetch once
        if self.activity_types:
            return self.activity_types

        logger.debug('Fetching activity types')
        resp = requests.get(URL_ACTIVITY_TYPES)
        if not resp.ok:
            raise GarminAPIException('Failed to retrieve activity types')

        # Store as a clean dict, mapping keys and lower case common name
        types = resp.json()
        self.activity_types = {t['typeKey']: t for t in types}

        logger.debug('Fetched {} activity types'.format(len(self.activity_types)))  # noqa
        return self.activity_types

    def set_activity_type(self, session, activity):
        """
        Update the activity type
        """
        assert activity.id is not None
        assert activity.type is not None

        # Load the corresponding type key on Garmin Connect
        types = self.load_activity_types()
        type_key = types.get(activity.type)
        if type_key is None:
            logger.error("Activity type '{}' not valid".format(activity.type))
            return False

        url = '{}/{}'.format(URL_ACTIVITY_BASE, activity.id)
        data = {
            'activityId': activity.id,
            'activityTypeDTO': type_key
        }
        headers = dict(self.common_headers)  # clone
        headers['X-HTTP-Method-Override'] = 'PUT'  # weird. again.
        res = session.post(url, json=data, headers=headers)
        if not res.ok:
            raise GarminAPIException('Activity type not set: {}'.format(res.content))  # noqa



import os.path
try:
    # Python 3
    from configparser import ConfigParser
except ImportError:
    # Python 2
    from ConfigParser import RawConfigParser as ConfigParser
from garmin_uploader import logger, CONFIG_FILE
from garmin_uploader.api import GarminAPI


class User(object):
    """
    Garmin Connect user model
    Authenticates through web api as a browser
    """
    def __init__(self, username=None, password=None):
        """
        ---- GC login credential order of precedence ----
        1) Credentials given on command line
        2) Credentials given in config file in current working directory
        3) Credentials given in config file in user's home directory

        Command line overrides all, config in cwd overrides config in home dir
        """
        # Authenticated API session
        self.session = None

        configCurrentDir = os.path.abspath(
            os.path.normpath('./' + CONFIG_FILE)
        )
        configHomeDir = os.path.expanduser(
            os.path.normpath('~/' + CONFIG_FILE)
        )

        if username and password:
            logger.debug('Using credentials from command line.')
            self.username = username
            self.password = password
        elif os.path.isfile(configCurrentDir):
            logger.debug('Using credentials from \'%s\'.' % configCurrentDir)
            config = ConfigParser()
            config.read(configCurrentDir)
            self.username = config.get('Credentials', 'username')
            self.password = config.get('Credentials', 'password')
        elif os.path.isfile(configHomeDir):
            logger.debug('Using credentials from \'%s\'.' % configHomeDir)
            config = ConfigParser()
            config.read(configHomeDir)
            self.username = config.get('Credentials', 'username')
            self.password = config.get('Credentials', 'password')
        else:
            cwd = os.path.abspath(os.path.normpath('./'))
            homepath = os.path.expanduser(os.path.normpath('~/'))
            raise Exception("'{}' file does not exist in current directory {}"
                            "or home directory {}.  Use login options.".format(
                                CONFIG_FILE, cwd, homepath))

    def authenticate(self):
        """
        Authenticate on Garmin API
        """
        logger.info('Try to login on GarminConnect...')
        logger.debug('Username: {}'.format(self.username))
        logger.debug('Password: {}'.format('*'*len(self.password)))

        api = GarminAPI()
        try:
            self.session = api.authenticate(self.username, self.password)
            logger.debug('Login Successful.')
        except Exception as e:
            logger.critical('Login Failure: {}'.format(e))
            return False

        return True



import logging
import platform


# Setup config file name
if platform.system() == 'Windows':
    CONFIG_FILE = 'gupload.ini'
else:
    CONFIG_FILE = '.guploadrc'

# Garmin file extensions
VALID_GARMIN_FILE_EXTENSIONS = ('.tcx', '.fit', '.gpx')
BINARY_FILE_FORMATS = ('.fit',)

# Setup common logger
logger = logging.getLogger('garmin_uploader')
logger.setLevel(level=logging.WARNING)
channel = logging.StreamHandler()
formatter = logging.Formatter('%(asctime)s [%(levelname)s] %(message)s')
channel.setFormatter(formatter)
logger.addHandler(channel)



from setuptools import setup


def requirements(path):
    with open(path) as f:
        return f.read().splitlines()


setup(
    name="garmin-uploader",
    version=open("VERSION").read().replace('\n', ''),
    author="Bastien Abadie",
    author_email="bastien@nextcairn.com",
    description="A tool to upload FIT, GPX, and TCX files"
                "to the Garmin Connect web site.",
    url="https://github.com/La0/garmin-uploader",
    license="GPL",
    keywords="GARMIN CONNECT GPS TCX GPX FIT UPLOAD UPLOADER",
    packages=['garmin_uploader'],
    install_requires=requirements('requirements.txt'),
    tests_require=requirements('requirements-tests.txt'),
    entry_points={
        'console_scripts': [
            'gupload = garmin_uploader.cli:main',
        ]
    },
    package_data={
        'garmin_uploader': ['*.txt'],
    },
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: End Users/Desktop",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Topic :: Utilities",
    ],
)
